package javax.swing.plaf.metal;

import java.awt.event.MouseEvent;
import javax.swing.Icon;
import java.awt.Rectangle;
import javax.swing.SwingConstants;
import java.beans.PropertyChangeEvent;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.basic.BasicInternalFrameTitlePane;
import javax.swing.event.MouseInputAdapter;
import javax.swing.LookAndFeel;
import javax.swing.plaf.UIResource;
import javax.swing.ActionMap;
import javax.swing.SwingUtilities;
import java.awt.Container;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.border.Border;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicInternalFrameUI;

public class MetalInternalFrameUI extends BasicInternalFrameUI
{
    private static final PropertyChangeListener metalPropertyChangeListener;
    private static final Border handyEmptyBorder;
    protected static String IS_PALETTE;
    private static String IS_PALETTE_KEY;
    private static String FRAME_TYPE;
    private static String NORMAL_FRAME;
    private static String PALETTE_FRAME;
    private static String OPTION_DIALOG;
    
    public MetalInternalFrameUI(JInternalFrame b) {
        super(b);
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalInternalFrameUI((JInternalFrame)component);
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        Object clientProperty = c.getClientProperty(MetalInternalFrameUI.IS_PALETTE_KEY);
        if (clientProperty != null) {
            this.setPalette((boolean)clientProperty);
        }
        this.stripContentBorder(this.frame.getContentPane());
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        this.frame = (JInternalFrame)c;
        Container contentPane = ((JInternalFrame)(JInternalFrame)c).getContentPane();
        if ((contentPane instanceof JComponent)) {
            JComponent component = (JComponent)contentPane;
            if (component.getBorder() == MetalInternalFrameUI.handyEmptyBorder) {
                component.setBorder(null);
            }
        }
        super.uninstallUI(c);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.frame.addPropertyChangeListener(MetalInternalFrameUI.metalPropertyChangeListener);
    }
    
    @Override
    protected void uninstallListeners() {
        this.frame.removePropertyChangeListener(MetalInternalFrameUI.metalPropertyChangeListener);
        super.uninstallListeners();
    }
    
    @Override
    protected void installKeyboardActions() {
        super.installKeyboardActions();
        ActionMap uiActionMap = SwingUtilities.getUIActionMap(this.frame);
        if (uiActionMap != null) {
            uiActionMap.remove("showSystemMenu");
        }
    }
    
    @Override
    protected void uninstallKeyboardActions() {
        super.uninstallKeyboardActions();
    }
    
    @Override
    protected void uninstallComponents() {
        this.titlePane = null;
        super.uninstallComponents();
    }
    
    private void stripContentBorder(Object o) {
        if ((o instanceof JComponent)) {
            JComponent component = (JComponent)o;
            Border border = component.getBorder();
            if (border == null || border instanceof UIResource) {
                component.setBorder(MetalInternalFrameUI.handyEmptyBorder);
            }
        }
    }
    
    @Override
    protected JComponent createNorthPane(JInternalFrame internalFrame) {
        return new MetalInternalFrameTitlePane(internalFrame);
    }
    
    private void setFrameType(String s) {
        if (s.equals(MetalInternalFrameUI.OPTION_DIALOG)) {
            LookAndFeel.installBorder(this.frame, "InternalFrame.optionDialogBorder");
            ((MetalInternalFrameTitlePane)this.titlePane).setPalette(false);
        }
        else if (s.equals(MetalInternalFrameUI.PALETTE_FRAME)) {
            LookAndFeel.installBorder(this.frame, "InternalFrame.paletteBorder");
            ((MetalInternalFrameTitlePane)this.titlePane).setPalette(true);
        }
        else {
            LookAndFeel.installBorder(this.frame, "InternalFrame.border");
            ((MetalInternalFrameTitlePane)this.titlePane).setPalette(false);
        }
    }
    
    public void setPalette(boolean palette) {
        if (palette) {
            LookAndFeel.installBorder(this.frame, "InternalFrame.paletteBorder");
        }
        else {
            LookAndFeel.installBorder(this.frame, "InternalFrame.border");
        }
        ((MetalInternalFrameTitlePane)this.titlePane).setPalette(palette);
    }
    
    @Override
    protected MouseInputAdapter createBorderListener(JInternalFrame internalFrame) {
        return new MetalInternalFrameUI.BorderListener1();
    }
    
    static void access$200(MetalInternalFrameUI metalInternalFrameUI, String frameType) {
        metalInternalFrameUI.setFrameType(frameType);
    }
    
    static void access$400(MetalInternalFrameUI metalInternalFrameUI, Object o) {
        metalInternalFrameUI.stripContentBorder(o);
    }
    
    static {
        metalPropertyChangeListener = new MetalInternalFrameUI.MetalPropertyChangeHandler();
        handyEmptyBorder = new EmptyBorder(0, 0, 0, 0);
        MetalInternalFrameUI.IS_PALETTE = "JInternalFrame.isPalette";
        MetalInternalFrameUI.IS_PALETTE_KEY = "JInternalFrame.isPalette";
        MetalInternalFrameUI.FRAME_TYPE = "JInternalFrame.frameType";
        MetalInternalFrameUI.NORMAL_FRAME = "normal";
        MetalInternalFrameUI.PALETTE_FRAME = "palette";
        MetalInternalFrameUI.OPTION_DIALOG = "optionDialog";
    }
    
    private static class MetalPropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            JInternalFrame internalFrame = (JInternalFrame)propertyChangeEvent.getSource();
            if (!(internalFrame.getUI() instanceof MetalInternalFrameUI)) {
                return;
            }
            MetalInternalFrameUI metalInternalFrameUI = (MetalInternalFrameUI)internalFrame.getUI();
            if (propertyName.equals(MetalInternalFrameUI.FRAME_TYPE)) {
                if ((propertyChangeEvent.getNewValue() instanceof String)) {
                    MetalInternalFrameUI.access$200(metalInternalFrameUI, (String)propertyChangeEvent.getNewValue());
                }
            }
            else if (propertyName.equals(MetalInternalFrameUI.IS_PALETTE_KEY)) {
                if (propertyChangeEvent.getNewValue() != null) {
                    metalInternalFrameUI.setPalette((boolean)propertyChangeEvent.getNewValue());
                }
                else {
                    metalInternalFrameUI.setPalette(false);
                }
            }
            else if (propertyName.equals("contentPane")) {
                MetalInternalFrameUI.access$400(metalInternalFrameUI, propertyChangeEvent.getNewValue());
            }
            return;
        }
    }
    
    private class BorderListener1 extends BasicInternalFrameUI.BorderListener implements SwingConstants
    {
        Rectangle getIconBounds() {
            int leftToRight = MetalUtils.isLeftToRight(MetalInternalFrameUI.this.frame);
            int x = leftToRight ? 5 : (MetalInternalFrameUI.this.titlePane.getWidth() - 5);
            Object o = null;
            Icon frameIcon = MetalInternalFrameUI.this.frame.getFrameIcon();
            if (frameIcon != null) {
                if (leftToRight == 0) {
                    x -= frameIcon.getIconWidth();
                }
                o = new Rectangle(x, MetalInternalFrameUI.this.titlePane.getHeight() / 2 - frameIcon.getIconHeight() / 2, frameIcon.getIconWidth(), frameIcon.getIconHeight());
                return o;
            }
            else {
                return o;
            }
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
            if (mouseEvent.getClickCount() == 2 && mouseEvent.getSource() == MetalInternalFrameUI.this.getNorthPane() && MetalInternalFrameUI.this.frame.isClosable() && !MetalInternalFrameUI.this.frame.isIcon()) {
                Rectangle iconBounds = this.getIconBounds();
                if (iconBounds != null && iconBounds.contains(mouseEvent.getX(), mouseEvent.getY())) {
                    MetalInternalFrameUI.this.frame.doDefaultCloseAction();
                }
                else {
                    super.mouseClicked(mouseEvent);
                }
            }
            else {
                super.mouseClicked(mouseEvent);
            }
        }
    }
}
